package cd.casic.module.machine.component;

import cd.casic.module.machine.dal.model.FileNode;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpATTRS;
import com.jcraft.jsch.SftpException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

@Slf4j
@Component
public class FileTreeComponent {

    private ChannelSftp sftp;
    //todo缓存过期还未设置
    private final Map<String, FileNode> directoryCache = new ConcurrentHashMap<>();
    private static final long CACHE_EXPIRE_TIME = 60 * 1000; // 缓存过期时间：1分钟
    private final Map<String, Long> cacheTimeStamp = new ConcurrentHashMap<>();

    // 将文件树转换为JSON友好的Map结构（仅一级目录）
    public static Map<String, Object> convertToMap(FileNode node) {
        Map<String, Object> map = new HashMap<>();
        map.put("name", node.getName());
        map.put("isDirectory", node.isDirectory());
        map.put("size", node.getSize());
        map.put("permissions", node.getPermissions());
        map.put("modifiedTime", node.getModifiedTime());
        // 仅添加直接子项，不递归
        if (node.isDirectory() && !node.getChildren().isEmpty()) {
            List<Map<String, Object>> children = node.getChildren().stream()
                .map(FileTreeComponent::convertToMap)
                .collect(Collectors.toList());
            map.put("children", children);
        }
        return map;
    }

    // 获取指定路径的直接子项（不递归）
    public List<ChannelSftp.LsEntry> listDirectChildren(String path) {
        if (sftp == null || !isSftpConnected()) {
            log.error("SFTP连接无效，无法列出文件");
            throw exception(CONNECTION_LOST);
        }

        List<ChannelSftp.LsEntry> entries = Collections.synchronizedList(new ArrayList<>());
        //定义ChannelSftp下LsEntrySelector接口中select方法
        ChannelSftp.LsEntrySelector selector = entry -> {
            entries.add(entry);
            return ChannelSftp.LsEntrySelector.CONTINUE;
        };

        try {
            sftp.ls(path, selector);
        } catch (SftpException e) {
            log.error("读取远程文件目录结构失败 [错误码: {}, 信息: {}]",
                e.id, e.getMessage(), e);
            throw exception(READ_REMOTE_DIRECTORY_FAIL);
        }
        return new ArrayList<>(entries);
    }

    // 创建文件节点（支持使用完整路径或文件名）
    private static FileNode createFileNode(String name, SftpATTRS attrs, boolean useFullPath) {
        String displayName = useFullPath ? name : name.substring(name.lastIndexOf('/') + 1);
        return new FileNode(
            displayName,
            attrs.isDir(),
            attrs.getSize(),
            attrs.getPermissionsString(),
            attrs.getMTime() * 1000L  // 转换为毫秒
        );
    }

    // 获取远程文件树的方法（仅展示下一级）
    public Map<String, Object> getRemoteFileTree(Session session, String path) {
        path = normalizePath(path);
        Channel channel = null;
        try {
            // 缓存有效性检查
            String cacheKey = "ls:" + path;
            long currentTime = System.currentTimeMillis();

            // 检查缓存是否存在且未过期
            if (directoryCache.containsKey(cacheKey) &&
                currentTime - cacheTimeStamp.getOrDefault(cacheKey, 0L) < CACHE_EXPIRE_TIME) {
                log.debug("从缓存获取目录内容: {}", path);
                // 构建缓存中的根节点
                FileNode root = directoryCache.get(cacheKey);
                return sortFileInfo(convertToMap(root));
            }

            // 打开SFTP通道
            channel = session.openChannel("sftp");
            channel.connect();
            sftp = (ChannelSftp) channel;

            // 先通过stat获取当前路径信息
            SftpATTRS rootAttrs = sftp.stat(path);
            FileNode root = createFileNode(path, rootAttrs, true);

            // 仅获取直接子项，不递归
            List<ChannelSftp.LsEntry> entries = listDirectChildren(path);
            //循环添加子节点
            for (ChannelSftp.LsEntry entry : entries) {
                String fileName = entry.getFilename();
                if (fileName.equals(".") || fileName.equals("..")) continue;
                SftpATTRS attrs = entry.getAttrs();
                FileNode childNode = createFileNode(fileName, attrs, false);
                root.addChild(childNode);
            }
            // 更新缓
            directoryCache.put(cacheKey, root);
            cacheTimeStamp.put(cacheKey, currentTime);
            return sortFileInfo(convertToMap(root));
        } catch (JSchException e) {
            log.error("SFTP通道创建或连接失败: {}", e.getMessage(), e);
            if (e.getMessage().contains("open")) {
                throw exception(CREATE_CHANEL_ERROR);
            } else {
                throw exception(CHANEL_CONNECT_FAIL);
            }
        } catch (SftpException e) {
            if (e.id == ChannelSftp.SSH_FX_NO_SUCH_FILE) {
                // 仅捕获路径不存在的错误
                log.error("路径不存在: {}", path);
                throw exception(PATH_NOT_EXISTS); // 自定义异常
            } else if (e.id == ChannelSftp.SSH_FX_PERMISSION_DENIED) {
                // 处理权限问题（非路径不存在）
                log.error("无路径访问权限: {}", path);
                throw exception(NO_PATH_PERMISSION);
            } else {
                log.error("获取目录内容失败: {}", e.getMessage(), e);
                throw exception(READ_REMOTE_DIRECTORY_FAIL);
            }
        } finally {
            // 确保资源释放
            if (sftp != null) {
                try {
                    sftp.disconnect();
                } catch (Exception ex) {
                    log.warn("关闭SFTP连接失败", ex);
                }
            }
            if (channel != null && channel.isConnected()) {
                channel.disconnect();
            }
        }
    }

    // 检查SFTP连接状态
    private boolean isSftpConnected() {
        if (sftp == null) return false;
        try {
            sftp.pwd();
            return true;
        } catch (Exception e) {
            log.warn("SFTP连接状态检查失败", e);
            return false;
        }
    }

    // 规范化路径：确保末尾有斜杠（根路径除外）
    private String normalizePath(String path) {
        // 移除多余的斜杠（多个连续的斜杠会被替换为一个）
        path = path.replaceAll("/+", "/");

        // 如果路径不为根路径且末尾没有斜杠，则添加斜杠
        if (!"/".equals(path) && !path.endsWith("/")) {
            path += "/";
        }

        return path;
    }

    /**
     * 对文件和目录信息进行排序（仅处理直接子级）
     *
     * @param fileInfoMap 文件/目录信息映射
     * @return 排序后的映射
     */
    public Map<String, Object> sortFileInfo(Map<String, Object> fileInfoMap) {
        // 检查是否包含子节点
        if (fileInfoMap.containsKey("children")) {
            List<Map<String, Object>> children = (List<Map<String, Object>>) fileInfoMap.get("children");
            // 对子节点列表进行排序
            if (children != null && !children.isEmpty()) {
                children.sort((a, b) -> {
                    // 获取isDirectory属性并比较
                    boolean isADirectory = (boolean) a.get("isDirectory");
                    boolean isBDirectory = (boolean) b.get("isDirectory");
                    // 目录排在文件前面
                    return Boolean.compare(isBDirectory, isADirectory);
                });
                // 更新排序后的子节点列表
                fileInfoMap.put("children", children);
            }
        }
        return fileInfoMap;
    }
}
